@@ -1,6 +1,6 @@
 =head1 NAME
 
-Crypt::Spritz - Spritz stream cipher/hash/MAC/AEAD/CSPRNG module
+Crypt::Spritz - Spritz stream cipher/hash/MAC/AEAD/CSPRNG family
 
 =head1 SYNOPSIS
 
@@ -12,6 +12,10 @@ Crypt::Spritz - Spritz stream cipher/hash/MAC/AEAD/CSPRNG module
  my $cipher = new Crypt::Spritz::Cipher::XOR $key, $iv;
  $ciphertext = $cipher->crypt ($cleartext);
 
+ my $cipher = new Crypt::Spritz::Cipher $key, $iv;
+ $ciphertext  = $cipher->encrypt ($cleartext);
+ # $cleartext = $cipher->decrypt ($ciphertext);
+
  my $hasher = new Crypt::Spritz::Hash;
  $hasher->add ($data);
  $digest = $hasher->finish;
@@ -20,15 +24,22 @@ Crypt::Spritz - Spritz stream cipher/hash/MAC/AEAD/CSPRNG module
  $hasher->add ($data);
  $mac = $hasher->finish;
 
+ my $prng = new Crypt::Spritz::PRNG $entropy;
+ $prng->add ($additional_entropy);
+ $keydata = $prng->get (32);
+
  my $aead = new Crypt::Spritz::AEAD::XOR $key;
  $aead->nonce ($counter);
  $aead->associated_data ($header);
  $ciphertext = $aead->crypt ($cleartext);
  $mac = $aead->mac;
 
- my $prng = new Crypt::Spritz::PRNG $entropy;
- $prng->add ($additional_entropy);
- $keydata = $prng->get (32);
+ my $aead = new Crypt::Spritz::AEAD $key;
+ $aead->nonce ($counter);
+ $aead->associated_data ($header);
+ $ciphertext  = $aead->encrypt ($cleartext);
+ # $cleartext = $aead->decrypt ($ciphertext);
+ $mac = $aead->mac;
 
 =head1 DESCRIPTION
 
@@ -57,11 +68,13 @@ The Spritz base class is not meant for end users. To make usage simpler
 and safer, a number of convenience classes are provided for typical
 end-user tasks:
 
-   encryption               - Crypt::Spritz::Cipher::XOR
+   random number generation - Crypt::Spritz::PRNG
    hashing                  - Crypt::Spritz::Hash
    message authentication   - Crypt::Spritz::MAC
+   encryption               - Crypt::Spritz::Cipher::XOR
+   encryption               - Crypt::Spritz::Cipher
    authenticated encryption - Crypt::Spritz::AEAD::XOR
-   random number generation - Crypt::Spritz::PRNG
+   authenticated encryption - Crypt::Spritz::AEAD
 
 =cut
 
@@ -69,7 +82,7 @@ package Crypt::Spritz;
 
 use XSLoader;
 
-$VERSION = '0.1';
+$VERSION = 0.2;
 
 XSLoader::load __PACKAGE__, $VERSION;
 
@@ -122,6 +135,12 @@ Creates and returns a new, initialised Spritz state.
 
 Initialises the Spritz state again, throwing away the previous state.
 
+=item $another_spritz = $spritz->clone
+
+Make an exact copy of the spritz state. This method can be called on all
+of the objects in this module, but is documented separately to give some
+cool usage examples.
+
 =item $spritz->update               # Update
 
 =item $spritz->whip ($r)            # Whip
@@ -137,8 +156,8 @@ called manually.
 
 =item $spritz->absorb ($I)          # Absorb
 
-Absorbs the given data into the state (usually used for key material, nonces, IVs
-messages to be hashed and so on).
+Absorbs the given data into the state (usually used for key material,
+nonces, IVs messages to be hashed and so on).
 
 =item $spritz->absorb_stop          # AbsorbStop
 
@@ -161,6 +180,196 @@ Squeezes out the requested number of bytes from the state - this is usually
 =back
 
 
+=head2 THE Crypt::Spritz::PRNG CLASS
+
+This class implements a Pseudorandom Number Generatore (B<PRNG>),
+sometimes also called a Deterministic Random Bit Generator (B<DRBG>). In
+fact, it is even cryptographically secure, making it a B<CSPRNG>.
+
+Typical usage as a random number generator involves creating a PRNG
+object with a seed of your choice, and then fetching randomness via
+C<get>:
+
+   # create a PRNG object, use a seed string of your choice
+   my $prng = new Crypt::Spritz::PRNG $seed;
+
+   # now call get as many times as you wish to get binary randomness
+   my $some_randomness = $prng->get (17);
+   my moree_randomness = $prng->get (5000);
+   ...
+
+Typical usage as a cryptographically secure random number generator is to
+feed in some secret entropy (32 octets/256 bits are commonly considered
+enough), for example from C</dev/random> or C</dev/urandom>, and then
+generate some key material.
+
+   # create a PRNG object
+   my $prng = new Crypt::Spritz::PRNG;
+
+   # seed some entropy (either via ->add or in the constructor)
+   $prng->add ($some_secret_highly_entropic_string);
+
+   # now call get as many times as you wish to get
+   # hard to guess binary randomness
+   my $key1 = $prng->get (32);
+   my $key2 = $prng->get (16);
+   ...
+
+   # for long running programs, it is advisable to
+   # reseed the PRNG from time to time with new entropy
+   $prng->add ($some_more_entropy);
+
+=over 4
+
+=item $prng = new Crypt::Spritz::PRNG [$seed]
+
+Creates a new random number generator object. If C<$seed> is given, then
+the C<$seed> is added to the internal state as if by a call to C<add>.
+
+=item $prng->add ($entropy)
+
+Adds entropy to the internal state, thereby hopefully making it harder
+to guess. Good sources for entropy are irregular hardware events, or
+randomness provided by C</dev/urandom> or C</dev/random>.
+
+The design of the Spritz PRNG should make it strong against attacks where
+the attacker controls all the entropy, so it should be safe to add entropy
+from untrusted sources - more is better than less if you need a CSPRNG.
+
+For use as PRNG, of course, this matters very little.
+
+=item $octets = $prng->get ($length)
+
+Generates and returns C<$length> random octets as a string.
+
+=back
+
+
+=head2 THE Crypt::Spritz::Hash CLASS
+
+This implements the Spritz digest/hash algorithm. It works very similar to
+other digest modules on CPAN, such as L<Digest::SHA3>.
+
+Typical use for hashing:
+
+   # create hasher object
+   my $hasher = new Crypt::Spritz::Hash;
+
+   # now feed data to be hashed into $hasher
+   # in as few or many calls as required
+   $hasher->add ("Some data");
+   $hasher->add ("Some more");
+
+   # extract the hash - the object is not usable afterwards
+   my $digest = $hasher->finish (32);
+
+=over 4
+
+=item $hasher = new Crypt::Spritz::Hash
+
+Creates a new hasher object.
+
+=item $hasher->add ($data)
+
+Adds data to be hashed into the hasher state. It doesn't matter whether
+you pass your data in in one go or split it up, the hash will be the same.
+
+=item $digest = $hasher->finish ($length)
+
+Calculates a hash digest of the given length and return it. The object
+cannot sensibly be used for further hashing afterwards.
+
+Typical digest lengths are 16 and 32, corresponding to 128 and 256 bit
+digests, respectively.
+
+=item $another_hasher = $hasher->clone
+
+Make an exact copy of the hasher state. This can be useful to generate
+incremental hashes, for example.
+
+Example: generate a hash for the data already fed into the hasher, by keeping
+the original hasher for further C<add> calls and calling C<finish> on a C<clone>.
+
+   my $intermediate_hash = $hasher->clone->finish;
+
+Example: hash 64KiB of data, and generate a hash after every kilobyte that
+is over the full data.
+
+   my $hasher = new Crypt::Spritz::Hash;
+
+   for (0..63) {
+      my $kib = "x" x 1024; # whatever data
+
+      $hasher->add ($kib);
+
+      my $intermediate_hash = $hasher->clone->finish;
+      ...
+   }
+
+These kind of intermediate hashes are sometimes used in communications
+protocols to protect the integrity of the data incrementally, e.g. to
+detect errors early, while still having a complete hash at the end of a
+transfer.
+
+=back
+
+
+=head2 THE Crypt::Spritz::MAC CLASS
+
+This implements the Spritz Message Authentication Code algorithm. It works
+very similar to other digest modules on CPAN, such as L<Digest::SHA3>, but
+implements an authenticated digest (like L<Digest::HMAC>).
+
+I<Authenticated> means that, unlike L<Crypt::Spritz::Hash>, where
+everybody can verify and recreate the hash value for some data, with a
+MAC, knowledge of the (hopefully) secret key is required both to create
+and to verify the digest.
+
+Typical use for hashing is almost the same as with L<Crypt::Spritz::MAC>,
+except a key (typically 16 or 32 octets) is provided to the constructor:
+
+   # create hasher object
+   my $hasher = new Crypt::Spritz::Mac $key;
+
+   # now feed data to be hashed into $hasher
+   # in as few or many calls as required
+   $hasher->add ("Some data");
+   $hasher->add ("Some more");
+
+   # extract the mac - the object is not usable afterwards
+   my $mac = $hasher->finish (32);
+
+=over 4
+
+=item $hasher = new Crypt::Spritz::MAC $key
+
+Creates a new hasher object. The C<$key> can be of any length, but 16 and
+32 (128 and 256 bit) are customary.
+
+=item $hasher->add ($data)
+
+Adds data to be hashed into the hasher state. It doesn't matter whether
+you pass your data in in one go or split it up, the hash will be the same.
+
+=item $mac = $hasher->finish ($length)
+
+Calculates a message code of the given length and return it. The object
+cannot sensibly be used for further hashing afterwards.
+
+Typical digest lengths are 16 and 32, corresponding to 128 and 256 bit
+digests, respectively.
+
+=item $another_hasher = $hasher->clone
+
+Make an exact copy of the hasher state. This can be useful to
+generate incremental macs, for example.
+
+See the description for the C<Crypt::Spritz::Hash::clone> method for some
+examples.
+
+=back
+
+
 =head2 THE Crypt::Spritz::Cipher::XOR CLASS
 
 This class implements stream encryption/decryption. It doesn't implement
@@ -209,7 +418,7 @@ be long enough to distinguish repeated uses of tghe same key.
 
 =item $cleartext = $cipher->crypt ($encrypted)
 
-Encrypt or decrypt a piece of a message. This cna be called as many times
+Encrypt or decrypt a piece of a message. This can be called as many times
 as you want, and the message can be split into as few or many pieces as
 required without affecting the results.
 
@@ -217,6 +426,15 @@ required without affecting the results.
 
 Same as C<crypt>, except it I<modifies the argument in-place>.
 
+=item $another_cipher = $cipher->clone
+
+Make an exact copy of the cipher state. This can be useful to cache states
+for reuse later, for example, to avoid expensive key setups.
+
+While there might be use cases for this feature, it makes a lot more sense
+for C<Crypt::Spritz::AEAD> and C<Crypt::Spritz::AEAD::XOR>, as they allow
+you to specify the IV/nonce separately.
+
 =item $constant_32 = $cipher->keysize
 
 =item $constant_64 = $cipher->blocksize
@@ -230,90 +448,29 @@ not a block cipher and already provides an appropriate mode.
 =back
 
 
-=head2 THE Crypt::Spritz::Hash CLASS
+=head2 THE Crypt::Spritz::Cipher CLASS
 
-This implements the Spritz digest/hash algorithm. It works very similar to
-other digest modules on CPAN, such as L<Digest::SHA3>.
+This class is pretty much the same as the C<Crypt::Spritz::Cipher::XOR>
+class, with two differences: first, it implements the "standard" Spritz
+encryption algorithm, and second, while this variant is easier to analyze
+mathematically, there is little else to recommend it for, as it is slower,
+and requires lots of code duplication code.
 
-Typical use for hashing:
+So unless you need to be compatible with another implementation that does
+not offer the XOR variant, stick to C<Crypt::Spritz::Cipher::XOR>.
 
-   # create hasher object
-   my $hasher = new Crypt::Spritz::Hash;
-
-   # now feed data to be hashed into $hasher
-   # in as few or many calls as required
-   $hasher->add ("Some data");
-   $hasher->add ("Some more");
-
-   # extract the hash - the object is not usable afterwards
-   my $digest = $hasher->finish (32);
+All the methods from C<Crypt::Spritz::Cipher::XOR> are available, except
+C<crypt>, which has been replaced by separate C<encrypt> and C<decrypt>
+methods:
 
 =over 4
 
-=item $hasher = new Crypt::Spritz::Hash
+=item $encrypted = $cipher->encrypt ($cleartext)
 
-Creates a new hasher object.
+=item $cleartext = $cipher->decrypt ($encrypted)
 
-=item $hasher->add ($data)
-
-Adds data to be hashed into the hasher state. It doesn't matter whether
-you pass your data in in one go or split it up, the hash will be the same.
-
-=item $digest = $hasher->finish ($length)
-
-Calculates a hash digest of the given length and return it. The object
-cannot sensibly be used for further hashing afterwards.
-
-Typical digest lengths are 16 and 32, corresponding to 128 and 256 bit
-digests, respectively.
-
-=back
-
-
-=head2 THE Crypt::Spritz::MAC CLASS
-
-This implements the Spritz Message Authentication Code algorithm. It works
-very similar to other digest modules on CPAN, such as L<Digest::SHA3>, but
-implements an authenticated digest (like L<Digest::HMAC>).
-
-I<Authenticated> means that, unlike L<Crypt::Spritz::Hash>, where
-everybody can verify and recreate the hash value for some data, with a
-MAC, knowledge of the (hopefully) secret key is required both to create
-and to verify the digest.
-
-Typical use for hashing is almost the same as with L<Crypt::Spritz::MAC>,
-except a key (typically 16 or 32 octets) is provided to the constructor:
-
-   # create hasher object
-   my $hasher = new Crypt::Spritz::Mac $key;
-
-   # now feed data to be hashed into $hasher
-   # in as few or many calls as required
-   $hasher->add ("Some data");
-   $hasher->add ("Some more");
-
-   # extract the mac - the object is not usable afterwards
-   my $mac = $hasher->finish (32);
-
-=over 4
-
-=item $hasher = new Crypt::Spritz::MAC $key
-
-Creates a new hasher object. The C<$key> can be of any length, but 16 and
-32 (128 and 256 bit) are customary.
-
-=item $hasher->add ($data)
-
-Adds data to be hashed into the hasher state. It doesn't matter whether
-you pass your data in in one go or split it up, the hash will be the same.
-
-=item $mac = $hasher->finish ($length)
-
-Calculates a message code of the given length and return it. The object
-cannot sensibly be used for further hashing afterwards.
-
-Typical digest lengths are 16 and 32, corresponding to 128 and 256 bit
-digests, respectively.
+Really the same as C<Crypt::Spritz::Cipher::XOR>, except you need separate
+calls and code for encryption and decryption.
 
 =back
 
@@ -402,7 +559,7 @@ random strings i that your random number generator might be hosed and
 generate the same randomness multiple times (randomness can be very hard
 to get especially on embedded devices).
 
-=item $aead->associated_data ($data)(
+=item $aead->associated_data ($data)
 
 Provide the associated data (cleartext data to be authenticated but not
 encrypted). This method I<must> be called I<after> C<nonce> and I<before>
@@ -421,7 +578,7 @@ general how to interpret the encrypted part of a message.
 
 =item $cleartext = $cipher->crypt ($encrypted)
 
-Encrypt or decrypt a piece of a message. This cna be called as many times
+Encrypt or decrypt a piece of a message. This can be called as many times
 as you want, and the message can be split into as few or many pieces as
 required without affecting the results, with one exception: All except the
 last call to C<crypt> needs to pass in a multiple of C<64> octets. The
@@ -431,70 +588,53 @@ last call to C<crypt> does not have this limitation.
 
 Same as C<crypt>, except it I<modifies the argument in-place>.
 
+=item $another_cipher = $cipher->clone
+
+Make an exact copy of the cipher state. This can be useful to cache states
+for reuse later, for example, to avoid expensive key setups.
+
+Example: set up a cipher state with a key, then clone and use it to
+encrypt messages with different nonces.
+
+   my $cipher = new Crypt::Spritz::AEAD::XOR $key;
+
+   my $message_counter;
+
+   for my $message ("a", "b", "c") {
+      my $clone = $cipher->clone;
+      $clone->nonce (pack "N", ++$message_counter);
+      $clone->associated_data ("");
+      my $encrypted = $clone->crypt ($message);
+      ...
+   }
+
 =back
 
 
-=head2 THE Crypt::Spritz::PRNG CLASS
+=head2 THE Crypt::Spritz::AEAD CLASS
 
-This class implements a Pseudorandom Number Generatore (B<PRNG>),
-sometimes also called a Deterministic Random Bit Generator (B<DRBG>). In
-fact, it is even cryptographically secure, making it a B<CSPRNG>.
+This class is pretty much the same as the C<Crypt::Spritz::AEAD::XOR>
+class, with two differences: first, it implements the "standard" Spritz
+encryption algorithm, and second, while this variant is easier to analyze
+mathematically, there is little else to recommend it for, as it is slower,
+and requires lots of code duplication code.
 
-Typical usage as a random number generator involves creating a PRNG
-object with a seed of your choice, and then fetching randomness via
-C<get>:
+So unless you need to be compatible with another implementation that does
+not offer the XOR variant, stick to C<Crypt::Spritz::AEAD::XOR>.
 
-   # create a PRNG object, use a seed string of your choice
-   my $prng = new Crypt::Spritz::PRNG $seed;
-
-   # now call get as many times as you wish to get binary randomness
-   my $some_randomness = $prng->get (17);
-   my moree_randomness = $prng->get (5000);
-   ...
-
-Typical usage as a cryptographically secure random number generator is to
-feed in some secret entropy (32 octets/256 bits are commonly considered
-enough), for example from C</dev/random> or C</dev/urandom>, and then
-generate some key material.
-
-   # create a PRNG object
-   my $prng = new Crypt::Spritz::PRNG;
-
-   # seed some entropy (either via ->add or in the constructor)
-   $prng->add ($some_secret_highly_entropic_string);
-
-   # now call get as many times as you wish to get
-   # hard to guess binary randomness
-   my $key1 = $prng->get (32);
-   my $key2 = $prng->get (16);
-   ...
-
-   # for long running programs, it is advisable to
-   # reseed the PRNG from time to time with new entropy
-   $prng->add ($some_more_entropy);
+All the methods from C<Crypt::Spritz::AEAD::XOR> are available, except
+C<crypt>, which has been replaced by separate C<encrypt> and C<decrypt>
+methods:
 
 =over 4
 
-=item $prng = new Crypt::Spritz::PRNG [$seed]
+=item $encrypted = $cipher->encrypt ($cleartext)
 
-Creates a new random number generator object. If C<$seed> is given, then
-the C<$seed> is added to the internal state as if by a call to C<add>.
+=item $cleartext = $cipher->decrypt ($encrypted)
 
-=item $prng->add ($entropy)
-
-Adds entropy to the internal state, thereby hopefully making it harder
-to guess. Good sources for entropy are irregular hardware events, or
-randomness provided by C</dev/urandom> or C</dev/random>.
-
-The design of the Spritz PRNG should make it strong against attacks where
-the attacker controls all the entropy, so it should be safe to add entropy
-from untrusted sources - more is better than less if you need a CSPRNG.
-
-For use as PRNG, of course, this matters very little.
-
-=item $octets = $prng->get ($length)
-
-Generates and returns C<$length> random octets as a string.
+Really the same as C<Crypt::Spritz::AEAD::XOR>, except you need separate
+calls and code for encryption and decryption, but you have the same
+limitations on usage.
 
 =back
 
